@@ -17,8 +17,48 @@ setClass("DataFrame",
                    listData = structure(list(), names = character())),
          contains = c("DataTable", "SimpleList"))
 
+## Just a direct DataFrame extension with no additional slot for now. Once all
+## serialized DataFrame instances are replaced with DFrame instances (which
+## will take several BioC release cycles) we'll be able to move the DataFrame
+## slots from the DataFrame class definition to the DFrame class definition.
+## The final goal is to have DataFrame become a virtual class with no slots
+## that only extends DataTable, and DFrame a concrete DataFrame and SimpleList
+## subclass that has the same slots as the current DataFrame class.
+setClass("DFrame", contains="DataFrame")
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Accessor methods.
+### updateObject()
+###
+
+setMethod("updateObject", "DataFrame",
+    function(object, ..., verbose=FALSE)
+    {
+        ## class attribute.
+        if (class(object) == "DataFrame") {
+            ## Starting with S4Vectors 0.23.19, all DataFrame instances need
+            ## to be replaced with DFrame instances. Note that this is NOT a
+            ## change of the internals, only a change of the class attribute.
+            if (verbose)
+                message("[updateObject] Settting class attribute of DataFrame ",
+                        "instance to \"DFrame\" ... ", appendLF=FALSE)
+            class(object) <- class(new("DFrame"))
+            if (verbose)
+                message("OK")
+        } else {
+            if (verbose)
+                message("[updateObject] ", class(object), " object ",
+                        "is current.\n",
+                        "[updateObject] Nothing to update.")
+        }
+
+        callNextMethod()
+    }
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Accessors
 ###
 
 setMethod("nrow", "DataFrame", function(x) x@nrows)
@@ -80,10 +120,10 @@ setReplaceMethod("colnames", "DataFrame",
                    x
                  })
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Validity.
-###
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Validity
+###
 
 .valid.DataFrame.dim <- function(x)
 {
@@ -113,8 +153,20 @@ setReplaceMethod("colnames", "DataFrame",
   NULL
 }
 
+.OLD_DATAFRAME_INSTANCE_MSG <- c(
+    "Note that starting with BioC 3.10, the class attribute ",
+    "of all DataFrame **instances** should be set to ",
+    "\"DFrame\". Please update this object ",
+    "with 'updateObject(object, verbose=TRUE)' and ",
+    "re-serialize it."
+)
+
 .valid.DataFrame <- function(x)
 {
+  ## class() is broken when used within a validity method. See:
+  ##   https://stat.ethz.ch/pipermail/r-devel/2019-August/078337.html
+  #if (class(x) == "DataFrame")
+  #  return(paste(.OLD_DATAFRAME_INSTANCE_MSG, collapse=""))
   c(.valid.DataFrame.dim(x),
     .valid.DataFrame.rownames(x),
     .valid.DataFrame.names(x))
@@ -124,14 +176,16 @@ setValidity2("DataFrame", .valid.DataFrame)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Constructor.
+### Constructor
 ###
 
 ### Low-level constructor. For internal use only.
-### Note that, when supplied, 'nrows' is trusted.
-### Calling 'new_DataFrame(x)' on an ordinary named list 'x' will turn it
-### into a DataFrame in the most possibly straightforward way, that is,
-### calling 'as.list()' on the result will return a list identical to 'x'.
+### Note that, when supplied, 'nrows' is trusted (except when 'listData' is a
+### data.frame or data-frame-like object).
+### Calling 'new_DataFrame(x)' on an ordinary named list or an ordinary
+### data.frame 'x' will turn it into a DataFrame in the most possibly
+### straightforward way. In particular calling 'as.list()' or 'as.data.frame()'
+### on the returned DataFrame will bring back 'x'.
 ### This is unlike 'DataFrame(x)' or 'as(x, "DataFrame")' which can do all
 ### kind of hard-to-predict mangling to 'x', unless the user does something
 ### like 'DataFrame(lapply(x, I))'. Not super convenient or intuitive!
@@ -141,21 +195,34 @@ new_DataFrame <- function(listData=list(), nrows=NA, what="arguments")
         stopifnot(isSingleNumberOrNA(nrows))
         if (!is.integer(nrows))
             nrows <- as.integer(nrows)
-        if (length(listData) == 0L) {
-            if (is.na(nrows))
-                nrows <- 0L
-            names(listData) <- character(0)
-        } else {
-            if (is.na(nrows)) {
-                elt_nrows <- elementNROWS(listData)
-                nrows <- elt_nrows[[1L]]
-                if (!all(elt_nrows == nrows))
-                    stop(wmsg(what, " imply differing number of rows"))
+        listData_nrow <- nrow(listData)
+        if (is.null(listData_nrow)) {
+            ## 'listData' is NOT a data.frame or data-frame-like object.
+            if (length(listData) == 0L) {
+                if (is.na(nrows))
+                    nrows <- 0L
+                names(listData) <- character(0)
+            } else {
+                if (is.na(nrows)) {
+                    elt_nrows <- elementNROWS(listData)
+                    nrows <- elt_nrows[[1L]]
+                    if (!all(elt_nrows == nrows))
+                        stop(wmsg(what, " imply differing number of rows"))
+                }
+                if (is.null(names(listData)))
+                    names(listData) <- paste0("V", seq_along(listData))
             }
-            if (is.null(names(listData)))
-                names(listData) <- paste0("V", seq_along(listData))
+        } else {
+            ## 'listData' is a data.frame or data-frame-like object.
+            if (is.na(nrows)) {
+                nrows <- listData_nrow
+            } else if (nrows != listData_nrow) {
+                stop(wmsg("the supplied 'nrows' does not match ",
+                          "the nb of rows in 'listData'"))
+            }
+            listData <- as.list(listData)
         }
-        new2("DataFrame", nrows=nrows, listData=listData, check=FALSE)
+        new2("DFrame", nrows=nrows, listData=listData, check=FALSE)
 }
 
 DataFrame <- function(..., row.names = NULL, check.names = TRUE,
@@ -182,7 +249,7 @@ DataFrame <- function(..., row.names = NULL, check.names = TRUE,
     varnames[dotnames == ""] <- list(NULL)
     nrows <- ncols <- integer(length(varnames))
     for (i in seq_along(listData)) {
-      element <- try(as(listData[[i]], "DataFrame"), silent = TRUE)
+      element <- try(as(listData[[i]], "DFrame"), silent = TRUE)
       if (inherits(element, "try-error"))
         stop("cannot coerce class \"", class(listData[[i]])[1L],
              "\" to a DataFrame")
@@ -242,8 +309,9 @@ DataFrame <- function(..., row.names = NULL, check.names = TRUE,
   ans
 }
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Subsetting.
+### Subsetting
 ###
 
 setMethod("[[", "DataFrame", function(x, i, j, ...)
@@ -458,11 +526,12 @@ setMethod("replaceCOLS", c("DataFrame", "ANY"), function(x, i, value) {
 setMethod("normalizeSingleBracketReplacementValue", "DataFrame",
           function(value, x)
           {
-              isColumnList <- is(value, "DataFrame") || is.list(value)
-              if (is.null(value) || (isColumnList && length(value) == 0L))
+              hasColumns <- is(value, "DataFrame") || is.list(value) ||
+                  length(dim(value)) >= 2L
+              if (is.null(value) || (hasColumns && length(value) == 0L))
                   return(NULL)
               value <- as(value, "DataFrame", strict=FALSE)
-              if (!isColumnList) {
+              if (!hasColumns) {
                   names(value) <- NULL # don't try this at home
               }
               value
@@ -521,8 +590,9 @@ setMethod("rep", "DataFrame", function(x, ...) {
   x[rep(seq_len(nrow(x)), ...),,drop=FALSE]
 })
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Coercion.
+### Coercion
 ###
 
 ## Break DataFrame into a normal R data.frame
@@ -598,7 +668,7 @@ setMethod("as.matrix", "DataFrame", function(x) {
 })
 
 ## take data.frames to DataFrames
-setAs("data.frame", "DataFrame",
+setAs("data.frame", "DFrame",
       function(from) {
         rn <- attributes(from)[["row.names"]]
         if (is.integer(rn))
@@ -614,15 +684,15 @@ setAs("data.frame", "DataFrame",
         ans
       })
 
-setAs("data.table", "DataFrame",
+setAs("data.table", "DFrame",
     function(from)
     {
         df <- data.table:::as.data.frame.data.table(from)
-        as(df, "DataFrame")
+        as(df, "DFrame")
     }
 )
 
-setAs("table", "DataFrame",
+setAs("table", "DFrame",
       function(from) {
         df <- as.data.frame(from)
         factors <- sapply(df, is.factor)
@@ -631,19 +701,19 @@ setAs("table", "DataFrame",
       })
 
 setOldClass(c("xtabs", "table"))
-setAs("xtabs", "DataFrame",
+setAs("xtabs", "DFrame",
       function(from) {
         class(from) <- "table"
-        as(from, "DataFrame")
+        as(from, "DFrame")
       })
 
 .defaultAsDataFrame <- function(from) {
   if (length(dim(from)) == 2L) {
-    df <- as.data.frame(from)
+    df <- as.data.frame(from, stringsAsFactors=FALSE)
     if (0L == ncol(from))
       ## colnames on matrix with 0 columns are 'NULL'
       names(df) <- character()
-    as(df, "DataFrame")
+    as(df, "DFrame")
   } else {
     ans <- new_DataFrame(setNames(list(from), "X"), nrows=length(from))
     ans@rownames <- names(from)
@@ -651,7 +721,8 @@ setAs("xtabs", "DataFrame",
   }
 }
 
-setAs("ANY", "DataFrame", .defaultAsDataFrame)
+setAs("ANY", "DFrame", .defaultAsDataFrame)
+setAs("ANY", "DataFrame", function(from) as(from, "DFrame"))
 
 .VectorAsDataFrame <- function(from) {
   ans <- .defaultAsDataFrame(from)
@@ -662,24 +733,24 @@ setAs("ANY", "DataFrame", .defaultAsDataFrame)
 }
 
 ## overriding the default inheritance-based coercion from methods package
-setAs("SimpleList", "DataFrame", .VectorAsDataFrame)
-setAs("Vector", "DataFrame", .VectorAsDataFrame)
+setAs("SimpleList", "DFrame", .VectorAsDataFrame)
+setAs("Vector", "DFrame", .VectorAsDataFrame)
 
 ## note that any element named 'row.names' will be interpreted differently
 ## is this a bug or a feature?
-setAs("list", "DataFrame",
+setAs("list", "DFrame",
       function(from) {
         do.call(DataFrame, c(from, list(check.names=is.null(names(from)))))
       })
 
-setAs("NULL", "DataFrame", function(from) as(list(), "DataFrame"))
+setAs("NULL", "DFrame", function(from) as(list(), "DFrame"))
 
-setAs("AsIs", "DataFrame",
+setAs("AsIs", "DFrame",
       function(from) {
         new_DataFrame(setNames(list(drop_AsIs(from)), "X"))
       })
 
-setAs("ANY", "DataTable_OR_NULL", function(from) as(from, "DataFrame"))
+setAs("ANY", "DataTable_OR_NULL", function(from) as(from, "DFrame"))
 
 setMethod("coerce2", "DataFrame",
     function(from, to)
@@ -728,8 +799,32 @@ setMethod("coerce2", "DataFrame",
     }
 )
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Combining.
+### Display
+###
+
+setMethod("classNameForDisplay", "DFrame",
+    function(x) if (class(x) == "DFrame") "DataFrame" else class(x)
+)
+
+setMethod("show", "DataFrame",
+    function(object)
+    {
+        if (class(object) == "DataFrame") {
+            ## Aug 20, 2019: Too early for this warning.
+            #warning(wmsg(.OLD_DATAFRAME_INSTANCE_MSG))
+            object <- updateObject(object, check=FALSE)
+        }
+        invisible(callNextMethod())
+    }
+)
+
+setMethod("showAsCell", "DataFrame", showAsCell_array)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Combining
 ###
 
 ### Return an integer matrix with 1 column per object in 'objects' and 1 row
@@ -778,8 +873,8 @@ setMethod("coerce2", "DataFrame",
 ###     is an Rle. The wrapper below **mitigate** this by decoding the Rle
 ###     columns first. Note that this is a mitigation process only. For
 ###     example it will help if Rle columns are mixed with atomic vectors
-###     or factors, but it won't help if the cols[[1]] is an Rle and the
-###     other columns are IntegerList objects.
+###     or factors, but it won't help if cols[[1]] is an Rle and the other
+###     columns are IntegerList objects.
 ###  2) When the columns to bind are a mix of atomic vectors and factors,
 ###     bindROWS() would **always** return an atomic vector (whatever
 ###     cols[[1]] is, i.e. atomic vector or factor). However we **always**
