@@ -84,7 +84,7 @@ setMethod("na.exclude", "DataTable",
           })
 
 setMethod("is.na", "DataTable", function(x) {
-  na <- do.call(cbind, lapply(seq(ncol(x)), function(xi) is.na(x[[xi]])))
+  na <- do.call(cbind, lapply(seq(ncol(x)), function(xi) decode(is.na(x[[xi]]))))
   rownames(na) <- rownames(x)
   na
 })
@@ -280,7 +280,7 @@ setMethod("show", "DataTable",
               ntail <- get_showTailLines()
               nr <- nrow(object)
               nc <- ncol(object)
-              cat(class(object), " with ",
+              cat(classNameForDisplay(object), " with ",
                   nr, ifelse(nr == 1, " row and ", " rows and "),
                   nc, ifelse(nc == 1, " column\n", " columns\n"),
                   sep = "")
@@ -315,6 +315,7 @@ setMethod("show", "DataTable",
                   out <- rbind(classinfo, out)
                   print(out, quote = FALSE, right = TRUE)
               }
+              invisible(NULL)
           })
 
 .rownames <- function(nms, nrow, nhead, ntail)
