@@ -19,7 +19,7 @@ setClassUnion("list_OR_List", c("list", "List"))  # list-like objects
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Accessor methods.
+### Accessor methods
 ###
 
 setGeneric("elementType", function(x, ...) standardGeneric("elementType"))
@@ -68,37 +68,7 @@ setMethod("isEmpty", "List", function(x) all(elementNROWS(x) == 0L))
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### relistToClass()
-###
-### 'relistToClass(x)' is the opposite of 'elementType(y)' in the sense that
-### the former returns the class of the result of relisting (or splitting)
-### 'x' while the latter returns the class of the result of unlisting (or
-### unsplitting) 'y'.
-###
-### More formally, if 'x' is an object that is relistable and 'y' a list-like
-### object:
-###    relistToClass(x) == class(relist(x, some_skeleton))
-###    elementType(y) == class(unlist(y))
-###
-### As a consequence, for any object 'x' for which relistToClass() is defined
-### and returns a valid class, 'elementType(new(relistToClass(x)))' should
-### return 'class(x)'.
-###
-
-setGeneric("relistToClass", function(x) standardGeneric("relistToClass"))
-
-.selectListClassName <- function(x) {
-  cn <- listClassName("Compressed", x)
-  if (cn == "CompressedList")
-    cn <- listClassName("Simple", x)
-  cn
-}
-
-setMethod("relistToClass", "ANY", function(x) .selectListClassName(class(x)))
-
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Constructor.
+### Constructor
 ###
 
 List <- function(...)
@@ -111,7 +81,7 @@ List <- function(...)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### The "show" method.
+### Display
 ###
 
 setMethod("show", "List",
@@ -125,6 +95,22 @@ setMethod("show", "List",
           })
 
 
+.showAsCell_list <- function(object)
+{
+    vapply(object,
+        function(x) {
+            str <- paste(showAsCell(head(x, 3L)), collapse=",")
+            if (length(x) > 3L)
+                str <- paste0(str, ",...")
+            str
+        },
+        character(1L),
+        USE.NAMES=FALSE
+    )
+}
+setMethod("showAsCell", "list_OR_List", .showAsCell_list)
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### unlist()
 ###
@@ -211,7 +197,7 @@ setMethod("unlist", "List",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Subsetting.
+### Subsetting
 ###
 
 ### Assume 'x' and 'i' are parallel List objects (i.e. same length).
@@ -590,25 +576,9 @@ setMethod("setListElement", "List", setListElement_default)
 setMethod("getListElement", "List",
           function(x, i) getListElement(as.list(x), i))
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Simple helper functions for some common subsetting operations.
-###
-
-phead <- function(x, n=6L)
-{
-    .Defunct("IRanges::heads")
-    IRanges::heads(x, n=n)
-}
-
-ptail <- function(x, n=6L)
-{
-    .Defunct("IRanges::tails")
-    IRanges::tails(x, n=n)
-}
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Coercion.
+### Coercion
 ###
 
 setAs("List", "list", function(from) as.list(from))
