@@ -6,7 +6,6 @@
 ### sequence i.e. an ordered finite collection of elements.
 ###
 
-
 setClassUnion("DataTable_OR_NULL", c("DataTable", "NULL"))
 
 setClass("Vector",
@@ -17,6 +16,19 @@ setClass("Vector",
     )
 )
 
+### Beware that:
+###   > is(factor(), "vector_OR_Vector")
+###   [1] TRUE
+### even though:
+###   > is(factor(), "vector")
+###   [1] FALSE
+###   > is(factor(), "Vector")
+###   [1] FALSE
+### See R/S4-utils.R for other examples of messed up inheritance with union
+### classes.
+### TODO: Should we explicitely add "factor" to this union?
+setClassUnion("vector_OR_Vector", c("vector", "Vector"))  # vector-like objects
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### parallelSlotNames()
@@ -52,6 +64,30 @@ setMethod("parallelVectorNames", "ANY",
           function(x) setdiff(colnames(as.data.frame(new(class(x)))), "value"))
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### updateObject()
+###
+### The default method (defined in BiocGenerics) does complicated, costly,
+### and dangerous things, and sometimes it actually breaks valid objects
+### (e.g. it breaks valid OverlapEncodings objects). So we overwrite it with
+### a method for Vector objects that does nothing! That way it's simple,
+### cheap, and safe ;-). And that's really all it needs to do at the moment.
+### UPDATE: Starting with S4Vectors 0.23.19, all DataFrame instances need
+### to be replaced with DFrame instances. So the updateObject() method for
+### Vector objects got updated from doing nothing (no-op) to call
+### updateObject() on the elementMetadata component of the object.
+
+setMethod("updateObject", "Vector",
+    function(object, ..., verbose=FALSE)
+    {
+        ## Update from DataFrame to DFrame.
+        object@elementMetadata <- updateObject(object@elementMetadata,
+                                               ..., verbose=verbose)
+        object
+    }
+)
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Getters
 ###
@@ -87,7 +123,7 @@ setMethod("elementMetadata", "Vector",
     {
         if (!isTRUEorFALSE(use.names))
             stop("'use.names' must be TRUE or FALSE")
-        ans <- x@elementMetadata
+        ans <- updateObject(x@elementMetadata, check=FALSE)
         if (use.names && !is.null(ans))
             rownames(ans) <- names(x)
         ans
@@ -107,9 +143,9 @@ setGeneric("values", function(x, ...) standardGeneric("values"))
 
 setMethod("values", "Vector", function(x, ...) elementMetadata(x, ...))
 
-setMethod("anyNA", "Vector", function(x, recursive=FALSE) any(is.na(x)))
+setMethod("anyNA", "Vector", function(x, recursive=FALSE) FALSE)
 
-setMethod("is.na", "Vector", function(x) rep(FALSE, length(x)))
+setMethod("is.na", "Vector", function(x) rep.int(FALSE, length(x)))
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -204,19 +240,109 @@ setValidity2("Vector", .valid.Vector)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### updateObject()
-###
-### The default method (defined in BiocGenerics) does complicated, costly,
-### and dangerous things, and sometimes it actually breaks valid objects
-### (e.g. it breaks valid OverlapEncodings objects). So we overwrite it with
-### a method for Vector objects that does nothing! That way it's simple,
-### cheap, and safe ;-). And that's really all it needs to do at the moment.
+### Setters
 ###
 
-setMethod("updateObject", "Vector",
-    function(object, ..., verbose=FALSE) object
+setGeneric("elementMetadata<-",
+           function(x, ..., value) standardGeneric("elementMetadata<-"))
+
+### NOT exported but used in packages IRanges, GenomicRanges,
+### SummarizedExperiment, GenomicAlignments, and maybe more...
+### 3x faster than new("DFrame", nrows=nrow).
+### 500x faster than DataFrame(matrix(nrow=nrow, ncol=0L)).
+make_zero_col_DataFrame <- function(nrow) new_DataFrame(nrows=nrow)
+
+### NOT exported but used in the IRanges and GenomicRanges packages.
+normarg_mcols <- function(mcols, x_class, x_len)
+{
+    ## Note that 'mcols_target_class' could also be obtained with
+    ## 'getClassDef(x_class)@slots[["elementMetadata"]]', in which
+    ## case the class name would be returned with the "package" attribute.
+    mcols_target_class <- getSlots(x_class)[["elementMetadata"]]
+    ok <- is(mcols, mcols_target_class)
+    if (is.null(mcols)) {
+        if (ok)
+            return(mcols)  # NULL
+        mcols <- make_zero_col_DataFrame(x_len)
+    } else if (is.list(mcols)) {
+        ## Note that this will also handle an 'mcols' that is a data.frame
+        ## or a data.frame derivative (e.g. data.table object).
+        if (ok)
+            return(mcols)
+        mcols <- new_DataFrame(mcols)
+    } else {
+        mcols <- updateObject(mcols, check=FALSE)
+    }
+    ok <- is(mcols, mcols_target_class)
+    if (!ok)
+        mcols <- as(mcols, mcols_target_class)
+
+    ## From now on, 'mcols' is guaranteed to be a DataTable object.
+    if (!is.null(rownames(mcols)))
+        rownames(mcols) <- NULL
+
+    mcols_nrow <- nrow(mcols)
+    if (mcols_nrow == x_len)
+        return(mcols)
+    one <- ncol(mcols) == 1L
+    if (mcols_nrow > x_len && mcols_nrow != 1L)
+        stop(wmsg("trying to set ", if (one) "a " else "",
+                  "metadata column", if (one) "" else "s", " ",
+                  "of length ", mcols_nrow, " on an object of length ", x_len))
+    if (mcols_nrow == 0L)
+        stop(wmsg("trying to set ", if (one) "a " else "", "zero length ",
+                  "metadata column", if (one) "" else "s", " ",
+                  "on a non-zero length object "))
+    if (x_len %% mcols_nrow != 0L)
+        warning(wmsg("You supplied ", if (one) "a " else "",
+                     "metadata column", if (one) "" else "s", " ",
+                     "of length ", mcols_nrow, " to set on an object ",
+                     "of length ", x_len, ". However please note that ",
+                     "the latter is not a multiple of the former."))
+    i <- rep(seq_len(mcols_nrow), length.out=x_len)
+    extractROWS(mcols, i)
+}
+
+setReplaceMethod("elementMetadata", "Vector",
+    function(x, ..., value)
+    {
+        value <- normarg_mcols(value, class(x), length(x))
+        BiocGenerics:::replaceSlots(x, elementMetadata=value, check=FALSE)
+    }
 )
 
+setGeneric("mcols<-", function(x, ..., value) standardGeneric("mcols<-"))
+
+setReplaceMethod("mcols", "Vector",
+    function(x, ..., value) `elementMetadata<-`(x, ..., value=value)
+)
+
+setGeneric("values<-", function(x, ..., value) standardGeneric("values<-"))
+
+setReplaceMethod("values", "Vector",
+                 function(x, value) {
+                     elementMetadata(x) <- value
+                     x
+                 })
+
+setGeneric("rename", function(x, ...) standardGeneric("rename"))
+
+.renameVector <- function(x, ...) {
+  newNames <- c(...)
+  if (!is.character(newNames) || any(is.na(newNames))) {
+      stop("arguments in '...' must be character and not NA")
+  }
+  badOldNames <- setdiff(names(newNames), names(x))
+  if (length(badOldNames))
+    stop("Some 'from' names in value not found on 'x': ",
+         paste(badOldNames, collapse = ", "))
+  names(x)[match(names(newNames), names(x))] <- newNames
+  x
+}
+
+setMethod("rename", "vector", .renameVector)
+setMethod("rename", "Vector", .renameVector)
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Coercion
@@ -312,82 +438,6 @@ as.list.Vector <- function(x, ...) as.list(x, ...)
 setMethod("as.list", "Vector", function(x, ...) as.list(as(x, "List"), ...))
 
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Setters
-###
-
-setGeneric("elementMetadata<-",
-           function(x, ..., value) standardGeneric("elementMetadata<-"))
-
-### NOT exported but used in packages IRanges, GenomicRanges,
-### SummarizedExperiment, GenomicAlignments, and maybe more...
-### 3x faster than new("DataFrame", nrows=nrow).
-### 500x faster than DataFrame(matrix(nrow=nrow, ncol=0L)).
-make_zero_col_DataFrame <- function(nrow) new_DataFrame(nrows=nrow)
-
-.normalize_mcols_replacement_value <- function(value, x)
-{
-    x_slots <- getSlots(class(x))
-    ## Should never happen because 'x' should always be a Vector object so
-    ## should always have the 'elementMetadata' slot.
-    if (!("elementMetadata" %in% names(x_slots)))
-        stop(wmsg("trying to set metadata columns on an object that does ",
-                  "not support them (i.e. with no 'elementMetadata' slot)"))
-    target_class <- x_slots[["elementMetadata"]]
-    if (is.null(value)) {
-        if (is(NULL, target_class))
-            return(NULL)
-        value <- make_zero_col_DataFrame(length(x))
-    }
-    if (!is(value, target_class))
-        value <- as(value, target_class)
-    ## From here 'value' is guaranteed to be a DataTable object.
-    if (!is.null(rownames(value)))
-        rownames(value) <- NULL
-    V_recycle(value, x, x_what="value", skeleton_what="x")
-}
-
-setReplaceMethod("elementMetadata", "Vector",
-    function(x, ..., value)
-    {
-        value <- .normalize_mcols_replacement_value(value, x)
-        BiocGenerics:::replaceSlots(x, elementMetadata=value, check=FALSE)
-    }
-)
-
-setGeneric("mcols<-", function(x, ..., value) standardGeneric("mcols<-"))
-
-setReplaceMethod("mcols", "Vector",
-    function(x, ..., value) `elementMetadata<-`(x, ..., value=value)
-)
-
-setGeneric("values<-", function(x, ..., value) standardGeneric("values<-"))
-
-setReplaceMethod("values", "Vector",
-                 function(x, value) {
-                     elementMetadata(x) <- value
-                     x
-                 })
-
-setGeneric("rename", function(x, ...) standardGeneric("rename"))
-
-.renameVector <- function(x, ...) {
-  newNames <- c(...)
-  if (!is.character(newNames) || any(is.na(newNames))) {
-      stop("arguments in '...' must be character and not NA")
-  }
-  badOldNames <- setdiff(names(newNames), names(x))
-  if (length(badOldNames))
-    stop("Some 'from' names in value not found on 'x': ",
-         paste(badOldNames, collapse = ", "))
-  names(x)[match(names(newNames), names(x))] <- newNames
-  x
-}
-
-setMethod("rename", "vector", .renameVector)
-setMethod("rename", "Vector", .renameVector)
-
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Subsetting
 ###
@@ -541,6 +591,9 @@ setMethod("replaceROWS", c("Vector", "ANY"),
 subset.Vector <- function(x, ...) subset(x, ...)
 subset_Vector <- function(x, subset, select, drop=FALSE, ...)
 {
+    ## Fix old objects on-the-fly (e.g. old GRanges, GRangesList, or
+    ## GAlignments instances).
+    x <- updateObject(x, check=FALSE)
     i <- evalqForSubset(subset, x, ...)
     x_mcols <- mcols(x, use.names=FALSE)
     if (!is.null(x_mcols)) {
@@ -583,6 +636,25 @@ repROWS <- function(x, ...) extractROWS(x, rep(seq_len(NROW(x)), ...))
 setMethod("rep", "Vector", repROWS)
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Displaying
+###
+
+.Vector_summary <- function(object)
+{
+    object_len <- length(object)
+    object_mcols <- mcols(object, use.names=FALSE)
+    object_nmc <- if (is.null(object_mcols)) 0L else ncol(object_mcols)
+    paste0(classNameForDisplay(object), " object of length ", object_len,
+           " with ", object_nmc, " metadata ",
+           ifelse(object_nmc == 1L, "column", "columns"))
+}
+### S3/S4 combo for summary.Vector
+summary.Vector <- function(object, ...)
+    .Vector_summary(object, ...)
+setMethod("summary", "Vector", summary.Vector)
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Concatenation along the ROWS
 ###
@@ -729,7 +801,6 @@ setMethod("append", c("Vector", "Vector"),
     }
 )
 
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Evaluating
 ###
