@@ -13,9 +13,11 @@
 ###     >=
 ###     <
 ###     >
+###     sameAsPreviousROW
 ###     match
 ###     selfmatch
 ###     duplicated
+###     anyDuplicated
 ###     unique
 ###     %in%
 ###     findMatches
@@ -23,6 +25,7 @@
 ###     order
 ###     sort
 ###     rank
+###     xtfrm
 ###     table
 
 
@@ -40,6 +43,28 @@
 
 setGeneric("pcompare", function(x, y) standardGeneric("pcompare"))
 
+setMethod("pcompare", c("numeric", "numeric"), function(x, y) {
+    as.integer(sign(x - y))
+})
+
+setMethod("pcompare", c("ANY", "ANY"), function(x, y) {
+    combined <- bindROWS(x, list(y))
+    original <- c(seq_len(NROW(x)), seq_len(NROW(y)))
+    is.x <- rep(c(TRUE, FALSE), c(NROW(x), NROW(y)))
+
+    o <- order(combined)
+    original <- original[o]
+    is.x <- is.x[o]
+
+    grouping <- cumsum(!sameAsPreviousROW(extractROWS(combined, o)))
+    x.groups <- integer(NROW(x))
+    x.groups[original[is.x]] <- grouping[is.x]
+    y.groups <- integer(NROW(y))
+    y.groups[original[!is.x]] <- grouping[!is.x]
+
+    pcompare(x.groups, y.groups)
+})
+
 ### The methods below are implemented on top of pcompare().
 
 setMethods("==", .OP2_SIGNATURES,
@@ -61,6 +86,95 @@ setMethods("<", .OP2_SIGNATURES, function(e1, e2) { !(e2 <= e1) })
 setMethods(">", .OP2_SIGNATURES, function(e1, e2) { !(e1 <= e2) })
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Comparisons along the ROWS
+###
+### Provides a basic implementation of what is different between ROWS.
+###
+### The default "sameAsPreviousROW" method below is implemented on top
+### of ==.
+###
+
+setGeneric("sameAsPreviousROW",
+    function(x) standardGeneric("sameAsPreviousROW")
+)
+
+setMethod("sameAsPreviousROW", "ANY", function(x) {
+    if (NROW(x)==0) {
+        logical(0)
+    } else {
+        c(FALSE, tail(x, n=-1L) == head(x, n=-1L))
+    }
+})
+
+.nasafe_compare <- function(z, y) {
+    comp <- z==y
+    na.z <- is.na(z)
+    na.y <- is.na(y)
+    comp[na.z!=na.y] <- FALSE
+    comp[na.z & na.y] <- TRUE
+    comp
+}
+
+.atomic_sameAsPreviousROW <- function(x) {
+    if (NROW(x)==0) {
+        logical(0)
+    } else {
+        z <- head(x, n=-1L) 
+        y <- tail(x, n=-1L)
+        c(FALSE, .nasafe_compare(z, y))
+    }
+}
+
+setMethod("sameAsPreviousROW", "atomic", .atomic_sameAsPreviousROW)
+
+# Explicitly define this to avoid dispatching to the numeric method
+# and suffering the unnecessary is.nan() checks.
+setMethod("sameAsPreviousROW", "integer", .atomic_sameAsPreviousROW)
+
+.numeric_sameAsPreviousROW <- function(x) {
+    if (NROW(x)==0) {
+        logical(0)
+    } else {
+        z <- head(x, n=-1L) 
+        y <- tail(x, n=-1L)
+        comp <- .nasafe_compare(z, y)
+
+        # No need to test for '&' to set to TRUE here, 
+        # as NaN equality is covered by NA equality.
+        nan.z <- is.nan(z)
+        nan.y <- is.nan(y)
+        comp[nan.z!=nan.y] <- FALSE
+
+        c(FALSE, comp)
+    }
+}
+
+setMethod("sameAsPreviousROW", "numeric", .numeric_sameAsPreviousROW)
+
+setMethod("sameAsPreviousROW", "complex", .numeric_sameAsPreviousROW)
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### match()
+###
+### The default "match" method below is implemented on top of
+### selfmatch().
+###
+
+setMethod("match", c("Vector", "Vector"),
+    function(x, table, nomatch = NA_integer_, incomparables = NULL, ...) 
+{
+    # table goes first so it gets picked up by 'selfmatch'.
+    combined <- bindROWS(table, list(x))
+
+    # Do NOT use nomatch=nomatch here, we need the NAs as a marker.
+    ans <- selfmatch(combined, nomatch=NA_integer_, incomparables=incomparables)
+    ans <- tail(ans, NROW(x))
+    ans[is.na(ans) | ans > NROW(table)] <- nomatch
+    ans
+})
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### selfmatch()
 ###
@@ -82,6 +196,27 @@ setMethod("selfmatch", "factor", function(x, ..., incomparables = NULL) {
     else as.integer(x)
 })
 
+### Vector-based "selfmatch" method, slightly more efficient than match(x, x).
+setMethod("selfmatch", "Vector", function(x, nomatch = NA_integer_, incomparables = NULL, ...) {
+    if (NROW(x)==0L) return(integer(0))
+
+    g <- grouping(x)
+    ends <- attr(g, "ends")
+    starts <- c(1L, head(ends, -1L) + 1L)
+    first.of.kind <- g[starts]
+
+    if (!is.null(incomparables)) {
+        # %in% should call match() with incomparables=NULL,
+        # otherwise we get an infinite loop of S4 dispatch!
+        first.x <- extractROWS(x, first.of.kind)
+        first.of.kind[first.x %in% incomparables] <- nomatch
+    }
+
+    ans <- integer(NROW(x))
+    ans[g] <- rep(first.of.kind, ends - starts + 1L)
+    ans
+})
+
 ### 'selfmatch_mapping' must be an integer vector like one returned by
 ### selfmatch(), that is, values are non-NAs and such that any value 'v' in it
 ### must appear for the first time at *position* 'v'.
@@ -109,10 +244,11 @@ reverseSelfmatchMapping <- function(selfmatch_mapping)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### duplicated() & unique()
+### duplicated() & anyDuplicated() & unique()
 ###
 ### The "duplicated" method below is implemented on top of selfmatch().
-### The "unique" method below is implemented on top of duplicated().
+### The "anyDuplicated" and "unique" methods below are implemented on top
+### of duplicated().
 ###
 
 ### S3/S4 combo for duplicated.Vector
@@ -121,13 +257,13 @@ duplicated.Vector <- function(x, incomparables=FALSE, ...)
 
 .duplicated.Vector <- function(x, incomparables=FALSE, ...)
 {
-    if (!identical(incomparables, FALSE)) 
-        stop("the \"duplicated\" method for Vector objects ", 
+    if (!identical(incomparables, FALSE))
+        stop("the \"duplicated\" method for Vector objects ",
              "only accepts 'incomparables=FALSE'")
     args <- list(...)
     if ("fromLast" %in% names(args)) {
         fromLast <- args$fromLast
-        if (!isTRUEorFALSE(fromLast)) 
+        if (!isTRUEorFALSE(fromLast))
             stop("'fromLast' must be TRUE or FALSE")
         args$fromLast <- NULL
         if (fromLast)
@@ -143,11 +279,26 @@ duplicated.Vector <- function(x, incomparables=FALSE, ...)
 }
 setMethod("duplicated", "Vector", .duplicated.Vector)
 
+### S3/S4 combo for anyDuplicated.Vector
+anyDuplicated.Vector <- function(x, incomparables=FALSE, ...)
+    anyDuplicated(x, incomparables=incomparables, ...)
+.anyDuplicated.Vector <- function(x, incomparables=FALSE, ...)
+{
+    if (!identical(incomparables, FALSE))
+        stop("the \"anyDuplicated\" method for Vector objects ",
+             "only accepts 'incomparables=FALSE'")
+    any(duplicated(x, incomparables=incomparables, ...))
+}
+setMethod("anyDuplicated", "Vector", .anyDuplicated.Vector)
+
 ### S3/S4 combo for unique.Vector
 unique.Vector <- function(x, incomparables=FALSE, ...)
     unique(x, incomparables=incomparables, ...)
 .unique.Vector <- function(x, incomparables=FALSE, ...)
 {
+    if (!identical(incomparables, FALSE))
+        stop("the \"unique\" method for Vector objects ",
+             "only accepts 'incomparables=FALSE'")
     i <- !duplicated(x, incomparables=incomparables, ...)
     extractROWS(x, i)
 }
@@ -191,11 +342,11 @@ setGeneric("countMatches", signature=c("x", "table"),
 ###   Hits of length 4
 ###   queryLength: 5
 ###   subjectLength: 6
-###     queryHits subjectHits 
-###      <integer>   <integer> 
-###    1         4           4 
-###    2         3           5 
-###    3         5           5 
+###     queryHits subjectHits
+###      <integer>   <integer>
+###    1         4           4
+###    2         3           5
+###    3         5           5
 ###    4         2           6
 ### and the cost of ordering them would probably defeat the purpose of the
 ### "put the smallest object on the right" optimization trick.
@@ -318,6 +469,22 @@ setMethod("rank", "Vector",
 )
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### xtfrm()
+###
+### The method below is implemented on top of order().
+###
+
+setMethod("xtfrm", "Vector", function(x) {
+    o <- order(x)
+    y <- extractROWS(x, o)
+    is.unique <- !sameAsPreviousROW(y)
+    out.rank <- cumsum(is.unique)
+    out.rank[o] <- out.rank
+    out.rank
+})
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### table()
 ###
