@@ -39,6 +39,20 @@ selectSome <- function(obj, maxToShow = 5, ellipsis = "...",
 }
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### coolcat()
+###
+
+### Exported!
+coolcat <- function(fmt, vals=character(), exdent=2, ...)
+{
+    vals <- ifelse(nzchar(vals), vals, "''")
+    lbls <- paste(selectSome(vals), collapse=" ")
+    txt <- sprintf(fmt, length(vals), lbls)
+    cat(strwrap(txt, exdent=exdent, ...), sep="\n")
+}
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### labeledLine()
 ###
@@ -372,7 +386,7 @@ setGeneric("showAsCell", function(object) standardGeneric("showAsCell"))
 ### Should also work on any array-like object with more than 2 dimensions
 ### that supports "reshaping" via the dim() setter. Note that DelayedArray
 ### objects don't support this reshaping in general.
-.showAsCell_array <- function(object)
+showAsCell_array <- function(object)
 {
     if (length(dim(object)) > 2L) {
         ## Reshape 'object' as a 2D object.
@@ -393,27 +407,18 @@ setGeneric("showAsCell", function(object) standardGeneric("showAsCell"))
 
 .default_showAsCell <- function(object)
 {
-  ## Some objects like SplitDataFrameList have a "dim" method that
-  ## returns a non-MULL object (a matrix!) even though they don't have
-  ## an array-like semantic.
-  if (length(dim(object)) >= 2L && !is.matrix(dim(object)))
-    return(.showAsCell_array(object))
-  object_NROW <- NROW(object)
-  if (object_NROW == 0L)
-    return(character(0L))
-  attempt <- try(as.character(object), silent=TRUE)
-  if (!is(attempt, "try-error"))
-      return(attempt)
-  if (!is(object, "list_OR_List"))
-      return(rep.int("#####", object_NROW))
-  vapply(object,
-      function(x) {
-          str <- paste(showAsCell(head(x, 3L)), collapse=",")
-          if (length(x) > 3L)
-              str <- paste0(str, ",...")
-          str
-      },
-      character(1L))
+    ## Some objects like SplitDataFrameList have a "dim" method that
+    ## returns a non-MULL object (a matrix!) even though they don't have
+    ## an array-like semantic.
+    if (length(dim(object)) >= 2L && !is.matrix(dim(object)))
+        return(showAsCell_array(object))
+    object_NROW <- NROW(object)
+    if (object_NROW == 0L)
+        return(character(0L))
+    attempt <- try(as.character(object), silent=TRUE)
+    if (!is(attempt, "try-error"))
+        return(attempt)
+    rep.int("#####", object_NROW)
 }
 
 setMethod("showAsCell", "ANY", .default_showAsCell)
