@@ -343,6 +343,31 @@ DEFINE_CCALLABLE_STUB(LLongAE *, new_LLongAE,
 	(       buflength,        nelt,           val)
 )
 
+DEFINE_CCALLABLE_STUB(size_t, LLongAEAE_get_nelt,
+	(const LLongAEAE *aeae),
+	(                 aeae)
+)
+
+DEFINE_CCALLABLE_STUB(size_t, LLongAEAE_set_nelt,
+	(LLongAEAE *aeae, size_t nelt),
+	(           aeae,        nelt)
+)
+
+DEFINE_NOVALUE_CCALLABLE_STUB(LLongAEAE_extend,
+	(LLongAEAE *aeae, size_t new_buflength),
+	(           aeae,        new_buflength)
+)
+
+DEFINE_NOVALUE_CCALLABLE_STUB(LLongAEAE_insert_at,
+	(LLongAEAE *aeae, size_t at, LLongAE *ae),
+	(           aeae,        at,          ae)
+)
+
+DEFINE_CCALLABLE_STUB(LLongAEAE *, new_LLongAEAE,
+	(size_t buflength, size_t nelt),
+	(       buflength,        nelt)
+)
+
 DEFINE_CCALLABLE_STUB(size_t, CharAE_get_nelt,
 	(const CharAE *ae),
 	(              ae)
@@ -513,6 +538,20 @@ DEFINE_CCALLABLE_STUB(SEXP, find_interv_and_start_from_width,
 	(           x,     x_len,            width,     width_len)
 )
 
+/*
+ * Stubs for callables defined in raw_utils.c
+ */
+
+DEFINE_CCALLABLE_STUB(SEXP, extract_bytes_by_positions,
+	(const char *x, int x_len, const int *pos, int npos, int collapse, SEXP lkup),
+	(            x,     x_len,            pos,     npos,     collapse,      lkup)
+)
+
+DEFINE_CCALLABLE_STUB(SEXP, extract_bytes_by_ranges,
+	(const char *x, int x_len, const int *start, const int *width, int nranges, int collapse, SEXP lkup),
+	(            x,     x_len,            start,            width,     nranges,     collapse,      lkup)
+)
+
 /*
  * Stubs for callables defined in Hits_class.c
  */
