@@ -282,6 +282,20 @@ test_DataFrame_replace <- function() {
   sw1["NewCity","NewCol"] <- DataFrame(0)
   swiss1["NewCity","NewCol"] <- data.frame(0)
   checkIdentical(as.data.frame(sw1), swiss1)
+  sw1 <- sw[,1:2]; swiss1 <- swiss[,1:2]
+  sw1[,colnames(sw)[2:3]] <- sw[,2:3]
+  swiss1[,colnames(swiss)[2:3]] <- swiss[,2:3]
+  checkIdentical(as.data.frame(sw1), swiss1)
+  
+  sw1 <- sw; swiss1 <- swiss
+  sw1[FALSE] <- list()
+  checkIdentical(sw1, sw)
+  sw1[1L] <- list()
+  swiss1[1L] <- list()
+  checkIdentical(as.data.frame(sw1), swiss1)
+  sw1[1L] <- NULL
+  swiss1[1L] <- NULL
+  checkIdentical(as.data.frame(sw1), swiss1)
 
   sw1 <- sw
   mcols(sw1) <- DataFrame(id = seq_len(ncol(sw1)))
