@@ -71,6 +71,10 @@ static const R_CallMethodDef callMethods[] = {
 	CALLMETHOD_DEF(safe_strexplode, 1),
 	CALLMETHOD_DEF(svn_time, 0),
 
+/* raw_utils.c */
+	CALLMETHOD_DEF(C_extract_character_from_raw_by_positions, 4),
+	CALLMETHOD_DEF(C_extract_character_from_raw_by_ranges, 5),
+
 /* eval_utils.c */
 	CALLMETHOD_DEF(top_prenv, 2),
 	CALLMETHOD_DEF(top_prenv_dots, 1),
@@ -175,6 +179,11 @@ void R_init_S4Vectors(DllInfo *info)
 	REGISTER_CCALLABLE(_LLongAE_extend);
 	REGISTER_CCALLABLE(_LLongAE_insert_at);
 	REGISTER_CCALLABLE(_new_LLongAE);
+	REGISTER_CCALLABLE(_LLongAEAE_get_nelt);
+	REGISTER_CCALLABLE(_LLongAEAE_set_nelt);
+	REGISTER_CCALLABLE(_LLongAEAE_extend);
+	REGISTER_CCALLABLE(_LLongAEAE_insert_at);
+	REGISTER_CCALLABLE(_new_LLongAEAE);
 	REGISTER_CCALLABLE(_CharAE_get_nelt);
 	REGISTER_CCALLABLE(_CharAE_set_nelt);
 	REGISTER_CCALLABLE(_CharAE_extend);
@@ -216,6 +225,10 @@ void R_init_S4Vectors(DllInfo *info)
 	REGISTER_CCALLABLE(_check_integer_pairs);
 	REGISTER_CCALLABLE(_find_interv_and_start_from_width);
 
+/* raw_utils.c */
+	REGISTER_CCALLABLE(_extract_bytes_by_positions);
+	REGISTER_CCALLABLE(_extract_bytes_by_ranges);
+
 /* Hits_class.c */
 	REGISTER_CCALLABLE(_new_Hits);
 	REGISTER_CCALLABLE(_get_select_mode);
